import TokenStream from './tokenstream';

/**
 * @param {TokenStream} stream
 * @param {boolean} isConstant
 * @return {string}
 */
function compileNative(stream, isConstant) {
  let name = stream.read();

  let tokens = [];
  let token = stream.read();

  while (token && token !== '\n') {
    tokens.push(token);

    token = stream.read();
  }

  let decl = `
/**
 * ${isConstant ? 'constant ' : ''}native ${name} ${tokens.join(' ')}
 * 
 * @param {lua_State} L
 * @return {number}
 */
function ${name}(L) {
  console.warn('${name} was called but is not implemented :(');
  return 0;
}`;

  return {name, decl};
}

/**
 * @param {Array<string>} names
 * @return {string}
 */
function compileLinkage(names) {
  return `
/**
 * @param {lua_State} L
 */
export default function bindNatives(L) {\n${names.map((name) => `  lua_register(L, '${name}', ${name});\n`).join('\n')}\n}`;
}

/**
 * @param {string} jass
 * @return {string}
 */
export default function compileNativesLua(jass) {
  let stream = new TokenStream(jass);
  let names = [];
  let decls = [];
  let token;

  while ((token = stream.read()) !== undefined) {
    let isConstant = false;

    if (token === 'constant') {
      isConstant = true;

      token = stream.read();
    }

    if (token === 'native') {
      let {name, decl} = compileNative(stream, isConstant);

      names.push(name);
      decls.push(decl);
    }
  }

  return `${decls.join('\n')}\n${compileLinkage(names)}\n`;
}
